#title  "AlienBrain"
#author "Oles Shishkovtsov"

#include xray
#include "scripts\\iq.h"

export OnInitialize;
export OnVisGain;
export OnVisTargetLost;
export OnTargetReached;
export OnHear;

export tm_TargetLost;
export tm_Anim;

#define NS_STAY		0.0
#define NS_WALK		20.0
#define NS_RUN		0.0

#define TM_StayAnimation	0
#define TM_TargetLost		1
#define TM_Test			2

// Purpose: re-select target
int iAnimSeq;
void tm_Anim(IQ_Brain* brain)
{
	iAnimSeq++;
	if (iAnimSeq>14) iAnimSeq=0;
	switch (iAnimSeq)
	{
	case 0:		iqAnim_play_cycle(brain,"idle"); break;
	case 1:		iqAnim_play_cycle(brain,"walk_fwd"); break;
	case 2:		iqAnim_play_cycle(brain,"walk_back"); break;
	case 3:		iqAnim_play_cycle(brain,"walk_ls"); break;
	case 4:		iqAnim_play_cycle(brain,"walk_rs");  break;
	case 5:		iqAnim_play_cycle(brain,"run_fwd"); break;
	case 6:		iqAnim_play_cycle(brain,"run_back"); break;
	case 7:		iqAnim_play_cycle(brain,"run_ls"); break;
	case 8:		iqAnim_play_cycle(brain,"run_rs"); break;
	case 9:		iqAnim_play_cycle(brain,"crouch_idle"); break;
	case 10:	iqAnim_play_cycle(brain,"crouch_walk_fwd");  break;
	case 11:	iqAnim_play_cycle(brain,"crouch_walk_back"); break;
	case 12:	iqAnim_play_cycle(brain,"crouch_walk_ls");	break;
	case 13:	iqAnim_play_cycle(brain,"crouch_walk_rs");	break;
	case 14:	iqAnim_play_cycle(brain,"jump");			break;
	}
	iqTimer_init_rand(&(brain->Timers[TM_Test]),"tm_Anim",5,6);
}

int OnInitialize(IQ_Brain* brain)
{
	brain->EyeMode			= EYE_LookAround;
	brain->EyeHeight		= 1;
	brain->EyeFOV			= 120;
	brain->EyeRange			= 100;
	brain->EyeLookSpeed		= 0.4; // 4
	iqVisible_set_clsid		(&(brain->Visible[TCLASS_Primary]),"O_ACTOR");
	brain->Target			= NULL;
	brain->NaviMode			= NAVI_Stay;
	brain->NaviPrediction		= 0.1;
	brain->NaviSpeed		= NS_STAY;
	brain->AuralPerception		= 0.5;

	iAnimSeq = 0;
	iqTimer_init_rand(&(brain->Timers[TM_Test]),"tm_Anim",7,10);

	return sizeof(IQ_Brain);
};

void timersCancel(IQ_Brain* brain)
{
	brain->Timers[TM_StayAnimation].cProcedure	= 0;
	brain->Timers[TM_TargetLost].cProcedure		= 0;
}

// Purpose: re-select target
// Note:	'Target' may be automatically cleared if it becomes invisible
//			TargetPoint specifies last 'seen' position
void naviPrimary(IQ_Brain* brain)
{
//	msg("naviPrimary");
	brain->Target	= brain->Visible[TCLASS_Primary].nearestReachable;
	brain->NaviMode	= NAVI_TargetObject;
	brain->EyeMode	= EYE_LookAround; //EYE_LookTarget;
	brain->NaviSpeed= NS_RUN;
	timersCancel(brain);
}
void naviOptional(IQ_Brain* brain)
{
//	msg("naviOptional");
	brain->Target	= brain->Visible[TCLASS_Optional].nearestReachable;
	brain->NaviMode	= NAVI_TargetObject;
	brain->EyeMode	= EYE_LookAround;
	brain->NaviSpeed= NS_WALK;
	timersCancel(brain);
}
void naviStay	(IQ_Brain* brain)
{
//	msg("naviStay");
	brain->Target	= NULL;
	brain->NaviMode	= NAVI_Stay;
	brain->EyeMode	= EYE_LookAround;
	brain->NaviSpeed= 0;
	timersCancel(brain);
}

void OnVisGain		(IQ_Brain* brain, int group)
{
//	msg("OnVisGain");
	if (brain->Target) {
		switch (group) {
		case TCLASS_Primary:		// enemy
			// re-select target
			naviPrimary(brain);
			break;
		case TCLASS_Secondary:		// dangerous thing
			// do nothing
			break;
		case TCLASS_Optional:		// item
			// re-select target
			if (brain->TargetGroup != TCLASS_Primary) {
				// we was moving to item or something else
				// reselect target if better exists
				naviOptional(brain);
			}
			break;
		}
	} else {
		// no target at all select anything
		switch (group) {
		case TCLASS_Primary:		// enemy
			// run to it
			naviPrimary(brain);
			break;
		case TCLASS_Secondary:		// dangerous thing
			// do nothing
			break;
		case TCLASS_Optional:		// item
			// select target
			if (brain->NaviMode!=NAVI_TargetPoint) 
				naviOptional(brain);
			break;
		}
	}
}

// Target lost
void OnVisTargetLost(IQ_Brain* brain)
{
//	msg("OnVisTargetLost");
	if (brain->Visible[TCLASS_Primary].nearestReachable)
	{
		naviPrimary(brain);
		return;
	}

	// Move to last seen position
	brain->NaviMode	= NAVI_TargetPoint;
	brain->EyeMode	= EYE_LookAround;
	brain->NaviSpeed= NS_RUN;
}

void tm_TargetLost(IQ_Brain* brain)
{
//	msg("tm_TargetLost");
	// Cancel all timers
	timersCancel(brain);

	// Make decision what to do
	if (brain->Visible[TCLASS_Primary].nearestReachable)
	{
		naviPrimary(brain);
	} else {
		if (brain->Visible[TCLASS_Optional].nearestReachable)
		{
			naviOptional(brain);
		} else {
			// I don't know what to do
			naviStay	(brain);
//			msg("* I don't know what to do :(");
		}
	}
}

void OnTargetReached(IQ_Brain* brain)
{
//	msg("OnTargetReached");
	// What in hell we was doing?
	if (brain->NaviMode==NAVI_TargetPoint)
	{
		// For some reason we was moving to specified point
		//	1. Pre-defined actions/scenery
		//	2. We lost our target and was going to last 'seen' point
		
		//  Stay here for some time - than make decision
		naviStay(brain);
		iqTimer_init_rand(&(brain->Timers[TM_TargetLost]),"tm_TargetLost",1,3);
	} else {
		// Kill him!
		naviStay(brain);
		iqTimer_init_rand(&(brain->Timers[TM_TargetLost]),"tm_TargetLost",1,3);
		//msg("* I don't know what to do :(");
	}
}

// Purpose: re-select target / turn / move to destination
void OnHear			(IQ_Brain* brain, Vector* where)
{
	//msg("OnHear");
	if (brain->Target) return;
	
	brain->TargetPoint	= *where;
	brain->NaviMode		= NAVI_TargetPoint;
	brain->EyeMode		= EYE_LookAround;
	brain->NaviSpeed	= NS_WALK;
	timersCancel		(brain);
};
